package CatalystX::ExtJS;

our $VERSION = '0.05';

1;

__END__

=head1 NAME

CatalystX::ExtJS - Collection of modules for ExtJS

=head1 MODULES

=over

=item L<CatalystX::Controller::ExtJS::REST>

RESTful interface to your data objects.

=item L<CatalystX::Controller::ExtJS::ImageChooser>

Controller class for the ExtJS ImageChooser component.

=back

=head1 AUTHOR

Moritz Onken, C<onken@netcubed.de>

=head1 CONTRIBUTORS

Mario Minati

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2009 by Moritz Onken.

This is free software, licensed under:

  The (three-clause) BSD License

=cut